const fs = require('fs');
const express = require('express');
const session = require('express-session');
const MongoDBStore = require('connect-mongodb-session')(session);
const collection = require('./config');
require("dotenv").config();

const app = express();

const organizations = require("../organization.json")

app.use(express.json({ extended: true }));
app.use(express.urlencoded({ extended: true }));

app.set('view engine', 'ejs');
app.use(express.static("public"));

const store = new MongoDBStore({
    uri: process.env.MONGO_URI, 
    userCollection: 'sessions'
});

app.use(session({
    secret: 'your-secret-key',
    resave: false,
    saveUninitialized: false,
    store: store
}));

const org = {
    id: 3,
    name: "Test",
    yearStarted: 1955,
    description: "Test desc",
    NumberOfPeopleDonated: 0
}

app.get('/signup', (req, res) => {
    res.render('signup');
});

app.get('/', (req, res) => {
    res.render('login');
});

app.get('/organizations', (req, res) => {
    res.render('organization', {
        organizations,
        username: req.session.loggedInUser.username,
        balance: req.session.loggedInUser.balance,
    });
});

const isLoggedIn = (req, res, next) => {
    if (req.session.isAuth) {
        return next();
    } else {
        res.redirect('/'); // Redirect to login page if user is not logged in
    }
};

app.use((req, res, next) => {

    if (req.ip == '127.0.0.1') {
        req.session.isAdmin = true;
    }

    next();
});

app.post('/logout', (req, res) => {
    req.session.destroy((err) => {
        if (err) {
            return console.log(err);
        }
        res.redirect('/');
    });
});

app.get('/dashboard', (req, res) => {
    if (req.session.isAuth) {
        res.render('home', {
            username: req.session.loggedInUser.username,
            balance: req.session.loggedInUser.balance,
            organizations,
            money: req.session.balance
        });
    } else {
        res.redirect('/');
    }
});

app.post('/signup', async (req, res) => {

    const data = {
        name: req.body.name,
        password: req.body.password
    };

    const existingUser = await collection.findOne({ name: data.name });
    if (existingUser) {
        return res.status(400).send("User already exists");
    }
    else {
        const userdata = await collection.insertMany(data);
        console.log("Inserted users:", userdata);
        res.send("Register Successfully!");
    }
});

app.post('/login', async (req, res) => {
    try {
        const check = await collection.findOne({ name: req.body.username });
        if (!check) {
            res.status(400).send("User not found");
        } else {
            if (check.password === req.body.password) {
                req.session.isAuth = true;
                req.session.loggedInUser = {
                    username: req.body.username,
                    balance: check.balance
                };
                res.status(200).redirect('/dashboard');
            } else {
                res.status(400).send("Invalid Password");
            }
        }
    } catch (error) {
        res.send("Error: " + error.message);
    }
});

app.post('/donate', async (req, res) => {
    const organizationId = parseInt(req.body.organizationId);
    const amount = parseInt(req.body.amount);

    const organization = Object.values(organizations).find(org => org.id === organizationId);

    console.log(typeof (organization))
    console.log('Request Body:', req.body);

    if (!organization) {
        res.status(400).send('Invalid organization id');
        return;
    }

    if (amount === 20) {
        if (req.session.loggedInUser.balance >= 20) {
            await collection.updateOne({ name: req.session.loggedInUser.username }, { $set: { balance: req.session.loggedInUser.balance - 20 } });
            req.session.loggedInUser.balance -= 20;
            res.send(`Donated $20 to ${organization.name}. Redirecting...<script>setTimeout(() => { window.location.href = '/organizations'; }, 2000);</script>`);
        } else {
            res.status(400).send('Insufficient balance for $20 donation');
        }
    } else if (amount === 1000) {
        if (req.session.loggedInUser.balance >= 1000) {
            await collection.updateOne({ name: req.session.loggedInUser.username }, { $set: { balance: req.session.loggedInUser.balance - 1000 } });
            req.session.loggedInUser.balance -= 1000;
    
            // Read flag.txt and send its contents
            const fs = require('fs');
            fs.readFile('flag.txt', 'utf8', (err, data) => {
                if (err) {
                    console.error(err);
                    res.status(500).send('Error reading flag file');
                } else {
                    res.send(`Super DONATE! Thank you for your generous $1000 donation! Flag: ${data}`);
                }
            });
        } else {
            res.status(400).send('Insufficient balance for $1000 donation');
        }
    }
    //add count for organization
    for (const [key, value] of Object.entries(req.body)) {

        if (!organizations[key]) {
            organizations[key] = JSON.parse(JSON.stringify(org));
        }

        for (const [k, v] of Object.entries(value)) {
            if (k === 'NumberOfPeopleDonated') {
                organizations[key][k] += v;
            } else {
                organizations[key][k] = v;
            }
        }
    }
});

app.post('/addBalance', async (req, res) => {
    try {
        if (!req.session.isAuth) {
            throw new Error('User session not found');
        }
        if (!req.session.isAdmin) {
            throw new Error('User is not admin');
        } else {
            console.log("user is admin")
            req.session.loggedInUser.balance += req.body.money;
            await collection.updateOne({ name: req.session.loggedInUser.username }, { $set: { balance: req.session.loggedInUser.balance } });
            res.send('Money added');
            console.log('Request Body:', req.body);
        }
    } catch (error) {
        console.error('Error:', error.message);
        res.status(500).send('Internal Server Error');
    }
});

const PORT = process.env.PORT;

app.listen(PORT, () => {
    console.log("Server is running on port ", PORT);
});
